% NUMODE: ex:springpend
% evolution of spring pendulum
%

% Right hand side in Cartesian coordinates
Hq = @(q) ((norm(q)-1)*q/norm(q) + [0;1]);
Hp = @(p) (p);

f = @(y) [-Hq(y(3:4));Hp(y(1:2))];

% Right hand side in Polar coordinates
HP=@(p,q)1/2*(p(1)^2+q(1)^-2*p(2)^2)-q(1)*sin(q(2))+1/2*(q(1)-1)^2;
HPq_1=@(p,q) -1/q(1)^3*p(2)^2-sin(q(2))+(q(1)-1);
HPq_2=@(p,q) -q(1)*cos(q(2));

HPp_1=@(p,q) p(1);
HPp_2=@(p,q) 1/q(1)^2*p(2);

g = @(y) [-HPq_1(y(1:2),y(3:4));...
    -HPq_2(y(1:2),y(3:4)) ;...
    HPp_1(y(1:2),y(3:4));...
    HPp_2(y(1:2),y(3:4))];

% initial data in Cartesian coordinates
p0 = [0;0];
q0 = [cos(pi/6);-sin(pi/6)];
y0 = [p0;q0];

% initial data in Polar coordinates
Pp0=[0;0];
Pq0=[1;pi/6];
Py0 = [Pp0;Pq0];

% timestep
h = 0.3;

% initial values
P = p0;
Q = q0+0.5*h*p0;
PP = Pp0;
PQ = Pq0+0.5*h*Pp0;
y = y0;
Py=Py0;

% energy
maxEn=HP(Pp0,Pq0);
minEn=HP(Pp0,Pq0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ANIMATION

k = 3;
spt = 1/(4*k)+1/(2*k)*(0:2*k-1);
spv = [[0,spt,1];[0,0.1*sin(2*k*pi*spt),0]];

figsv = figure('name','Spring Pendulum','position',[0 0 1600 800]);
subplot(2,2,1);
t = 0;

for i=1:1000
  subplot(2,2,1); q = Q; plotspring;
  subplot(2,2,2); q = y(3:4); plotspring;
  subplot(2,2,3); q = [PQ(1)*cos(PQ(2)) -PQ(1)*sin(PQ(2))]; plotspring;
  subplot(2,2,4); q = [Py(3)*cos(Py(4)) -Py(3)*sin(Py(4))]; plotspring;
  
  % symplectic Euler, Cartesian coordinates
  P = P - h*Hq(Q);
  Q = Q + h*Hp(P);
  
  % symplectic Euler, Polar coordinates
  PP(2)=PP(2)-h*HPq_2(PP,PQ);
  PP(1)=PP(1)-h*HPq_1(PP,PQ);
  PQ(2)=PQ(2)+h*HPp_2(PP,PQ);
  PQ(1)=PQ(1)+h*HPp_1(PP,PQ);
  
  % Explicit trapezoidal rule, Cartesian coordinates
  k1 = y + h*f(y);
  y = y + 0.5*h*(f(y)+f(k1));
  
  % Explicit trapezoidal rule. Polar coordinates
  k1 = Py + h*g(Py);
  Py = Py + 0.5*h*(g(Py)+g(k1));
  HP(PP,PQ)
  maxEn=max([maxEn,HP(PP,PQ)]);
  minEn=min([minEn,HP(PP,PQ)]);

  t = t+h;
end

[maxEn minEn]

